/*
** $Id: agnhlps.h $
** C Helper routines
** See Copyright Notice in agena.h
** initiated Alexander Walz, July 20, 2007
*/

#ifndef agnhlps_h
#define agnhlps_h


#include <stdio.h>

#ifndef LUA_DOS
  #include <stdint.h>     /* for int32_t */
  #include <sys/types.h>  /* for off_t & off64_t */
  #ifndef off64_t
    #define off64_t off_t
  #endif
  #ifndef lseek64
    #define lseek64 lseek
  #endif
#else
  #ifndef off_t
    #define off_t long
  #endif
  #ifndef int32_t
    #define int32_t long
  #endif
  #ifndef uint64_t
    #define uint64_t  unsigned long long
  #endif
  #ifndef off64_t
    #define off64_t	long long
  #endif
  #ifndef ssize_t
    #define ssize_t	long
  #endif
#endif

/* misc functions */

double tools_intpow (double x, int n);

/* used in packages `ads` */

void tools_quicksort (off64_t v[], int left, int right);

/* Dr. F.H.Toor's and my database C functions used in packages `base` and `phonetiqs` */

int my_open (const char *file);
int my_roopen (const char *file);
int my_create (const char *file) ;
int my_close (int hnd);
int my_seek (int hnd, int32_t pos);
off_t my_lof (int hnd);
void my_read (int hnd, void *data, size_t size);
int32_t my_readl (int hnd);
char my_readc (int hnd);
void my_write (int hnd, void *data, size_t size);
void my_writel (int hnd, int32_t data);
void my_writec (int hnd, char data);
int my_lock (int hnd, off64_t start, off64_t size);
int my_unlock (int hnd, off64_t start, off64_t size);
void my_move (int hnd, int32_t fpos, int32_t tpos, int32_t size);
void my_expand (int hnd, int mrc, int cnt, int count);
off64_t my_fpos (int hnd);

size_t sec_read (int hnd, void *data, size_t size);
size_t sec_readl (int hnd, ssize_t *success);

/* get keystroke function, does not work in Haiku */
#if defined(__unix__) || defined(__APPLE__)
int getch(void);
#endif

#if defined(__unix__)
int kbhit(void);
#endif

const char *tools_cwd (void);

/* Endian functions */

char tools_endian (void);

uint64_t tools_swapdouble (double n);
double tools_unswapdouble (uint64_t n);
void tools_swaplong (long *n);
void tools_swapint32_t (int32_t *n);

/* IEEE simulators */

int tools_isfinite (const double x);
int tools_isnan (const double x);
double tools_nan (void);


/* some constants */

#undef PI
#define PI (3.14159265358979323846)

#undef _E
#define _E (2.71828182845904523536)

#undef AGN_EPSILON
#define AGN_EPSILON  1.4901161193847656e-08  /* suited for doubles */

#undef RADIANS_PER_DEGREE
#define RADIANS_PER_DEGREE (PI/180.0)

#undef EULERGAMMA
#define EULERGAMMA (0.57721566490153286061)

double csgn (double a, double b);

char *charreplace (char *s, char from, char to);

/* char *chardelete (char *s, char from); */

/* code taken from: @(#)s_nextafter.c 5.1 93/09/24
 *
 * ====================================================
 * Copyright (C) 1993 by Sun Microsystems, Inc. All rights reserved.
 *
 * Developed at SunPro, a Sun Microsystems, Inc. business.
 * Permission to use, copy, modify, and distribute this
 * software is freely granted, provided that this notice
 * is preserved.
 * ====================================================
 *
 * IEEE functions
 *   nextafter(x,y)
 *   return the next machine floating-point number of x in the direction toward y.
 *   Special cases:
 *
 * Leave this code here and do not move it to agnconf.h etc. for incorrect results would be returned.
 *
 */

/* Get two 32 bit ints from a double.  */

#define EXTRACT_WORDS(ix0,ix1,d) \
do { \
  ieee_double_shape_type ew_u; \
  ew_u.value = (d); \
  (ix0) = ew_u.parts.msw; \
  (ix1) = ew_u.parts.lsw; \
} while (0)

#define GET_HI(ix0,d) \
do { \
  ieee_double_shape_type ew_u; \
  ew_u.value = (d); \
  (ix0) = ew_u.parts.msw; \
} while (0)

#define GET_LO(ix1,d) \
do { \
  ieee_double_shape_type ew_u; \
  ew_u.value = (d); \
  (ix1) = ew_u.parts.lsw; \
} while (0)

/* Set a double from two 32 bit ints.  */

#define INSERT_WORDS(d,ix0,ix1) \
do { \
  ieee_double_shape_type iw_u; \
  iw_u.parts.msw = (ix0); \
  iw_u.parts.lsw = (ix1); \
  (d) = iw_u.value; \
} while (0)

#endif


